/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*            (c) 2014 - 2018 SEGGER Microcontroller GmbH             *
*                                                                    *
*           www.segger.com     Support: support@segger.com           *
*                                                                    *
**********************************************************************
*                                                                    *
* All rights reserved.                                               *
*                                                                    *
* Redistribution and use in source and binary forms, with or         *
* without modification, are permitted provided that the following    *
* conditions are met:                                                *
*                                                                    *
* - Redistributions of source code must retain the above copyright   *
*   notice, this list of conditions and the following disclaimer.    *
*                                                                    *
* - Neither the name of SEGGER Microcontroller GmbH                  *
*   nor the names of its contributors may be used to endorse or      *
*   promote products derived from this software without specific     *
*   prior written permission.                                        *
*                                                                    *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND             *
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,        *
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF           *
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE           *
* DISCLAIMED.                                                        *
* IN NO EVENT SHALL SEGGER Microcontroller GmbH BE LIABLE FOR        *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  *
* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;    *
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      *
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE  *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
* DAMAGE.                                                            *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Generic application start

*/

#include "stm32f4xx.h"
#include "FreeRTOS.h"		// must be declared before task.h
#include "task.h"
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

TaskHandle_t xTaskHandle1 = NULL;
TaskHandle_t xTaskHandle2 = NULL;

void vTask1_handler(void *params);
void vTask2_handler(void *params);
static void prvSetupHardware(void);
static void prvSetupUART(void);
void printMsgUART(char *);

#ifdef USE_SEMIHOSTING
	extern void initialise_monitor_handles();
#endif

// Macros
#define TRUE 1
#define FALSE 0
#define AVAILABLE TRUE
#define NOT_AVAILABLE FALSE

// Global variables
uint8_t UART_ACCESS_KEY = AVAILABLE;

/*********************************************************************
*
*       main()
*
*  Function description
*   Application entry point.
*/
//void main(void) {
//  int i;
//
//  for (i = 0; i < 100; i++) {
//    printf("Hello World %d!\n", i);
//  }
//  do {
//    i++;
//  } while (1);
//}

void main(void)
{
//#ifdef USE_SEMIHOSTING
	//initialise_monitor_handles();
	printf("This is hello world example code\n");
//#endif

//	DWT->CTRL |= (1 << 0);			// Enable CCYCCNT register in DWT_CTRL for SEGGER

	// Reset RCC clock configuration to default reset state.
	// HSI ON, PLL OFF, HSE OFF, SYSCLK = 16 MHz, CPU_CLOCK = 16 MHz
	RCC_DeInit();

	// Update SyystemCoreClock variable
	SystemCoreClockUpdate();

	prvSetupHardware();

	printMsgUART("Application starting\n");

	// Start SEGGER recording
	SEGGER_SYSVIEW_Conf();
	SEGGER_SYSVIEW_Start();

	// Create two tasks
	xTaskCreate(vTask1_handler, "Task-1", configMINIMAL_STACK_SIZE, NULL, 2, &xTaskHandle1);

	xTaskCreate(vTask2_handler, "Task-2", configMINIMAL_STACK_SIZE, NULL, 2, &xTaskHandle2);

	// Start the schedule
	vTaskStartScheduler();

	for(;;);
}

void vTask1_handler(void *params)
{

	while(1)
	{
//#ifdef USE_SEMIHOSTING
//		printf("1");
//#endif
		if(UART_ACCESS_KEY == AVAILABLE)
		{
			UART_ACCESS_KEY = NOT_AVAILABLE;
//                        printf("1");
			printMsgUART("Task1\n");
			UART_ACCESS_KEY = AVAILABLE;
			taskYIELD();
		}
	}
}

void vTask2_handler(void *params)
{

	while(1)
	{
//#ifdef USE_SEMIHOSTING
//		printf("Task2\n");
//#endif
		if(UART_ACCESS_KEY == AVAILABLE)
		{
			UART_ACCESS_KEY = NOT_AVAILABLE;
//                        printf("2");
			printMsgUART("Task2\n");
			UART_ACCESS_KEY = AVAILABLE;
			taskYIELD();
		}
	}
}

static void prvSetupUART(void)
{
	GPIO_InitTypeDef gpio_uart_pins;
	USART_InitTypeDef uart2_init;

	// Enable UART peripheral clock
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

	// PA2 = USART2_TX, PA3 = USART2_RX
	// Alternate function configuration
	memset(&gpio_uart_pins, 0, sizeof(gpio_uart_pins));				// zeros out the structure
	gpio_uart_pins.GPIO_Mode = GPIO_Mode_AF;
	gpio_uart_pins.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3;
	gpio_uart_pins.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOA, &gpio_uart_pins);

	// AF mode setting for the pin
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_USART2);		// PA2
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_USART2);		// PA3

	// USART parameter init
	memset(&uart2_init, 0, sizeof(uart2_init));						// zeros out the structure
	uart2_init.USART_BaudRate = 115200;
	uart2_init.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	uart2_init.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	uart2_init.USART_Parity = USART_Parity_No;
	uart2_init.USART_StopBits = USART_StopBits_1;
	uart2_init.USART_WordLength = USART_WordLength_8b;
	USART_Init(USART2, &uart2_init);

	// Enable the UART peripheral
	USART_Cmd(USART2, ENABLE);
}

static void prvSetupHardware(void)
{
	prvSetupUART();													// Setup UART2
}

void printMsgUART(char *msg)
{
	for(uint32_t i = 0; i < strlen(msg); i++)
	{
		while (USART_GetFlagStatus(USART2, USART_FLAG_TXE) != SET);
		USART_SendData(USART2, msg[i]);
	}
}

/*************************** End of file ****************************/


